(function() {
  var Attributes, StringType, Type, aliases, isFloat, isInt, isInteger, isNumber, isString, register;

  isInt = require('util-ex/lib/is/string/int');

  isFloat = require('util-ex/lib/is/string/float');

  isNumber = require('util-ex/lib/is/type/number');

  isString = require('util-ex/lib/is/type/string');

  isInteger = require('util-ex/lib/is/type/integer');

  Attributes = require('abstract-type/lib/attributes');

  Type = require('abstract-type');

  register = Type.register;

  aliases = Type.aliases;

  module.exports = StringType = (function() {
    register(StringType);

    aliases(StringType, 'string');

    function StringType() {
      return StringType.__super__.constructor.apply(this, arguments);
    }

    StringType.prototype.$attributes = Attributes({
      min: {
        type: 'Number',
        assigned: '_min',
        assign: function(value, dest) {
          if (dest instanceof StringType) {
            value = dest.validateMin(value);
          }
          return value;
        }
      },
      max: {
        type: 'Number',
        assigned: '_max',
        assign: function(value, dest) {
          if (dest instanceof StringType) {
            value = dest.validateMax(value);
          }
          return value;
        }
      }
    });

    StringType.prototype.validateMin = function(value) {
      if (value > this._max) {
        throw new TypeError('the min should be less than max:' + value);
      }
      try {
        if (isString(value)) {
          value = parseInt(value);
        }
      } catch (_error) {}
      if (!isInteger(value)) {
        throw new TypeError('the min should be an integer');
      }
      return value;
    };

    StringType.prototype.validateMax = function(value) {
      if (value < this._min) {
        throw new TypeError('the max should be greater than min:' + value);
      }
      try {
        if (isString(value)) {
          value = parseInt(value);
        }
      } catch (_error) {}
      if (!isInteger(value)) {
        throw new TypeError('the max should be an integer');
      }
      return value;
    };

    StringType.prototype._isValid = function(value) {
      return isString(value);
    };

    StringType.prototype._validate = function(aValue, aOptions) {
      var result, vMax, vMin;
      result = this._isValid(aValue);
      if (result) {
        if (aOptions) {
          vMin = aOptions.min;
          vMax = aOptions.max;
          aValue = aValue.length;
          if (vMin != null) {
            result = aValue >= vMin;
            if (!result) {
              this.error('should be equal or greater than minimum length: ' + vMin);
            }
          }
          if (result && (vMax != null)) {
            result = aValue <= vMax;
            if (!result) {
              this.error('should be equal or less than maximum length: ' + vMax);
            }
          }
        }
      }
      return result;
    };

    return StringType;

  })();

}).call(this);

//# sourceMappingURL=index.js.map
